import { backKeyMap, frontKeyMap } from './js/fieldParamMap'
import { regularMap } from './js/regularCheck'
import hex_sha1 from './js/sha1.js'
import hex_md5 from './js/md5.js'
export default {
  // 判断字符串是否为json格式的字符串，返回 Boolean
  isJsonString(str) {
    try {
      if (typeof JSON.parse(str) === 'object') {
        return true
      } else {
        return false
      }
    } catch (e) {
      return false
    }
  },
  // 将有可能是json字符串的变量转化，如果不是json字符串原值返回
  parseJsonString(str) {
    try {
      if (typeof JSON.parse(str) === 'object') {
        return JSON.parse(str)
      } else {
        return str
      }
    } catch (e) {
      return str
    }
  },
  // 生成13位全局唯一id
  guid() {
    // 'xxxxxxxx-xxxx-3xxx-yxxx-xxxxxxxxxxxx' 36位
    // 'xxxxxxxx-3xxx' 13位
    return 'xxxxxxxx-3xxx'.replace(/[xy]/g, function(c) {
      var r = Math.random() * 16 | 0
      var v = c == 'x' ? r : (r & 0x3 | 0x8)
      return v.toString(16)
    })
  },
  // 专门处理 后端返回的“字段数组”
  handleBackFields(fields, callback) {
    if (!(fields instanceof Array)) {
      return
    }
    // 按fieldOrder属性升序
    fields.sort((a, b) => {
      return a.fieldOrder - b.fieldOrder
    })
    // 遍历后端返回的fields，将每个字段的属性名转化为前端的的属性名，并将属性值解析（包容json字符串）
    return fields.map((field, index) => {
      Object.keys(field).forEach(key => {
        if (backKeyMap[key] !== undefined && key !== backKeyMap[key]) {
          field[backKeyMap[key]] = this.parseJsonString(field[key])
          delete field[key]
        } else if (key === backKeyMap[key]) {
          field[backKeyMap[key]] = this.parseJsonString(field[key])
        }
      })
      if (field.type === 'select') {
        field.multiple = field.value instanceof Array
      }
      // 设置必填验证错误提示的变量、默认为false
      field.isError = false
      // TODO: 改变“表单设计器”传值，再去除此处判断转换
      // 针对某些 值为1或0的字段值 转换为Boolean值
      field.required = Number(field.required) === 1
      field.showLabel = (field.showLabel !== undefined && field.showLabel !== null) ? (Number(field.showLabel) > 0) : undefined
      field.showHeader = (field.showHeader !== undefined && field.showHeader !== null) ? (Number(field.showHeader) > 0) : undefined
      field.becomeFilter = (field.becomeFilter !== undefined && field.becomeFilter !== null) ? (Number(field.becomeFilter) > 0) : undefined
      field.becomeSortCondition = (field.becomeSortCondition !== undefined && field.becomeSortCondition !== null) ? (Number(field.becomeSortCondition) > 0) : undefined
      callback && callback(field, index)
      return field
    })
  },
  // 专门处理 前端提交的“字段数组”
  handleFrontFields(fields, callback) {
    if (!(fields instanceof Array)) {
      return
    }

    // 遍历后端返回的fields，将每个字段的属性名转化为前端的的属性名，并将属性值解析（包容json字符串）
    return fields.map((field, index) => {
      // 针对某些 值为1或0的字段值 转换为Boolean值
      field.required = field.required !== undefined
        ? (field.required ? 1 : 0) : undefined
      field.showLabel = field.showLabel !== undefined
        ? (field.showLabel ? 1 : 0) : undefined
      field.showHeader = field.showHeader !== undefined
        ? (field.showHeader ? 1 : 0) : undefined
      field.becomeFilter = field.becomeFilter !== undefined
        ? (field.becomeFilter ? 1 : 0) : undefined
      field.becomeSortCondition = field.becomeSortCondition !== undefined
        ? (field.becomeSortCondition ? 1 : 0) : undefined
      callback && callback(field, index)
      Object.keys(field).forEach(key => {
        if (frontKeyMap[key] !== undefined && key !== frontKeyMap[key]) {
          field[frontKeyMap[key]] = typeof field[key] === 'string' ? field[key] : JSON.stringify(field[key])
          delete field[key]
        } else if (key === backKeyMap[key]) {
          field[backKeyMap[key]] = typeof field[key] === 'string' ? field[key] : JSON.stringify(field[key])
        }
      })

      return field
    })
  },
  // 对字段数组进行提交前的验证
  validateFields(that, fields, pushEvent, initError) {
    // 为分组提供初始化赋值，防止分组将前面的error判断重置为false
    let isError = initError !== undefined ? initError : false

    // 判断字段是否为：null,空字符串,undefined,空数组
    function fieldIsEmpty(field) {
      return field.value === '' || field.value === null || field.value === undefined || (field.value instanceof Array && field.value.length < 1)
    }

    // 判断字段是否需要正则验证
    function fieldIsRegular(field) {
      return field.ruleRegular !== '' && field.ruleRegular !== null && field.ruleRegular !== undefined
    }
    // 遍历字段数组，进行验证、同时回掉pushEvent将字段放入数组中（可能存在不合法的字段）
    fields.forEach((field) => {
      // 如果字段为分组类型，则验证其子字段数组(递归处理)；
      if (field.type === 'group') {
        let groupId = ''
        if (field.value !== undefined && field.value instanceof Array && field.value.length > 0) {
          groupId = field.value[0].parentId
          isError = this.validateFields(that, field.value, pushEvent, isError)
        }
        const groupObj = Object.assign({}, field)
        groupObj.value = groupId
        pushEvent && pushEvent(groupObj)
      }
      // 如果字段可编辑,验证规则是否正确、放入要提交的数组中；
      // 不可编辑时，可能该类字段暂时不需要填写或者已经填写了，所以这里应该没必要再做规则检查。
      else if (field.isEdit !== '0') {
        // 如果字段值未填写（null,空字符串,undefined,空数组）,只验证该字段是否必填，且即使不必填也不做提交
        if (fieldIsEmpty(field)) {
          // 如果字段必填，则应针对“未填写”报错提示；
          // 不必填，则允许未填写、不做处理。
          if (field.required) {
            // 将errInfo置为空，使错误提示显示为“placeholder”
            that.$set(field, 'errInfo', '')
            field.isError = true
            isError = true
          }
          // 非必填时，只改变字段内的错误标识为false
          else {
            field.isError = false
          }
        }
        // 字段已填值，只验证该字段是否需要正则验证
        else {
          // 如果字段需要前端正则验证，则应针对“已填写”的值进行正则验证；
          if (fieldIsRegular(field)) {
            // 正则通过，只改变字段内的错误标识为false,但不能改变整体的错误标识（因为前面的字段可能存在错误）
            if (new RegExp(field.ruleRegular).test(field.value)) {
              field.isError = false
            }
            // 正则不通过， 字段内的错误标识和整体的错误标识都置为true
            else {
              if (regularMap[field.ruleRegular]) {
                that.$set(field, 'errInfo', '请输入正确的' + regularMap[field.ruleRegular])
              }
              field.isError = true
              isError = true
            }
          }
          // 不需要正则验证，则只改变字段内的错误标识为false
          else {
            field.isError = false
          }
        }
        // 无论是否对错，都放入要提交的数组中，因为只要字段中有错、后面提交前的整体错误标识都会拦截，下次即会清空、重新添加、直到数组中的字段无误
        pushEvent && pushEvent(field)
      }
    })
    return isError
  },
  // 返回 Unix毫秒时间戳 和 加密签名
  generateSignAndTimestamp() {
    // Unix毫秒时间戳
    const timestamp = Math.round(new Date().getTime() / 1000)

    // 应用私钥  4d8e605fa7ed546c4bcb33dee1381179
    const app_secret = '4d8e605fa7ed546c4bcb33dee1381179'

    // 服务端身份认证静态秘钥
    const serverAuthenticateStaticKey = 'de5kio2f'

    const sha1_key = hex_sha1(app_secret + timestamp + serverAuthenticateStaticKey)
    const sign = hex_md5(sha1_key)
    return {
      timestamp,
      sign
    }
  }
}
